Package: sgd
Type: Package
Title: Stochastic Gradient Descent for Scalable Estimation
Version: 1.1.3
Authors@R: c(
    person("Junhyung Lyle", "Kim", email = "jlylekim@gmail.com", role = c("cre", "aut")),
    person("Dustin", "Tran", role = "aut"),
    person("Panos", "Toulis", role = "aut"),
    person("Tian", "Lian", role = "ctb"),
    person("Ye", "Kuang", role = "ctb"),
    person("Edoardo", "Airoldi", role = "ctb")
    )
Maintainer: Junhyung Lyle Kim <jlylekim@gmail.com>
Description: A fast and flexible set of tools for large scale estimation. It
    features many stochastic gradient methods, built-in models, visualization
    tools, automated hyperparameter tuning, model checking, interval estimation,
    and convergence diagnostics.
URL: https://github.com/airoldilab/sgd
BugReports: https://github.com/airoldilab/sgd/issues
License: GPL-2
Imports: ggplot2, MASS, methods, Rcpp (>= 0.11.3), stats
Suggests: bigmemory, glmnet, gridExtra, R.rsp, testthat, microbenchmark
LinkingTo: BH, bigmemory, Rcpp, RcppArmadillo
LazyData: yes
VignetteBuilder: R.rsp
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2025-10-21 04:16:50 UTC; jlylekim
Author: Junhyung Lyle Kim [cre, aut],
  Dustin Tran [aut],
  Panos Toulis [aut],
  Tian Lian [ctb],
  Ye Kuang [ctb],
  Edoardo Airoldi [ctb]
Repository: CRAN
Date/Publication: 2025-10-21 11:00:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 10:18:57 UTC; unix
Archs: sgd.so.dSYM
