# Generated by roxygen2: do not edit by hand

S3method(coef,sgd)
S3method(fitted,sgd)
S3method(plot,list)
S3method(plot,sgd)
S3method(predict,sgd)
S3method(print,sgd)
S3method(residuals,sgd)
S3method(sgd,big.matrix)
S3method(sgd,default)
S3method(sgd,formula)
S3method(sgd,matrix)
export(predict_all)
export(sgd)
import(MASS)
importFrom(Rcpp,evalCpp)
importFrom(methods,new)
importFrom(stats,coef)
importFrom(stats,fitted)
importFrom(stats,gaussian)
importFrom(stats,is.empty.model)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,predict)
importFrom(stats,rnorm)
useDynLib(sgd)
